/***************************************************************************
 *   Copyright (C) 2006 by bysk                                           *
 *   buysk@pf.pl                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "led_notify.h"
#include "chat_manager.h"
#include "main_configuration_window.h"
#include "configuration_window_widgets.h"

#include <X11/Xlib.h>
#include <qmessagebox.h>

namespace
{
	const int NUMLOCK_LED = 1;
	const int CAPSLOCK_LED = 2;
	const int SCROLLLOCK_LED = 3;
} // namespace

LedNotify* led_notify_session;

extern "C" int led_notify_init()
{
	kdebugf();
	led_notify_session = new LedNotify();
	kdebugf2();
	return 0;
}

extern "C" void led_notify_close()
{
	kdebugf();
	delete led_notify_session;
	led_notify_session = NULL;
	kdebugf2();
}

LedBlinking::LedBlinking()
:
	timer_( new QTimer(this) ),
	value_( false ),
	enabled_( false ),
	countdown_( false ),
	delay_( config_file.readNumEntry("Led Notify", "LEDdelay") ),
	blinkCount_( config_file.readNumEntry("Led Notify", "LEDcount") ),
	counter_( 0 )
{
	connect( timer_, SIGNAL(timeout()), this, SLOT(blink()) );
}

LedBlinking::~LedBlinking()
{
	disconnect( timer_, SIGNAL(timeout()), this, SLOT(blink()) );
	if( value_ )
		setLed( SCROLLLOCK_LED, false );
}

void LedBlinking::startInfinite()
{
	if( !enabled_ || countdown_ )
	{
		countdown_ = false;
		enabled_ = true;
		blink();
	}
}

void LedBlinking::startFinite()
{
	if( !enabled_ || countdown_ )
	{
		countdown_ = true;
		enabled_ = true;
		counter_ = blinkCount_;
		blink();
	}
}

void LedBlinking::stop()
{
	enabled_ = false;
}

void LedBlinking::setLed( int led, bool highlight )
{
	static Display* theDisplay = QPaintDevice::x11AppDisplay();
	static XKeyboardControl values;
	
	values.led = led;
	values.led_mode = (highlight ? LedModeOn : LedModeOff);

	XChangeKeyboardControl(theDisplay, KBLed | KBLedMode, &values);
}

void LedBlinking::configurationUpdated()
{
	delay_ = config_file.readNumEntry("Led Notify", "LEDdelay");
	blinkCount_ = config_file.readNumEntry("Led Notify", "LEDcount");
}

void LedBlinking::blink(void)
{
	value_ = !value_;
	setLed( SCROLLLOCK_LED, value_ );
	
	// Count all "on" states
	if( countdown_ && value_ )
	{
		if( counter_ <= 1 )
			enabled_ = false;
		else
			--counter_;
	}
	
	// Make next blink() when:
	//	- blinkin is enabled
	//	- LED is "on" (we must turn it off)
	if( enabled_ || value_ )
	{
		timer_->start( delay_, TRUE );
	}
}

LedNotify::LedNotify(QObject* parent, const char* name)
:
	Notifier(parent, name)
{
	config_file.addVariable("Led Notify", "LEDdelay", 500);
	config_file.addVariable("Led Notify", "LEDcount", 3);
	config_file.addVariable("Notify", "NewChat_LED", false);
	config_file.addVariable("Notify", "NewMessage_LED", false);
	
	MainConfigurationWindow::registerUiFile(dataPath("kadu/modules/configuration/led_notify.ui"), this);
	notification_manager->registerNotifier( "Scroll Lock LED", this );
	connect( &pending, SIGNAL(messageFromUserDeleted(UserListElement)), this, SLOT(messageReceived(UserListElement)) );
}

LedNotify::~LedNotify()
{
	disconnect( &pending, SIGNAL(messageFromUserDeleted(UserListElement)), this, SLOT(messageReceived(UserListElement)) );
	notification_manager->unregisterNotifier("Scroll Lock LED");
	MainConfigurationWindow::unregisterUiFile(dataPath("kadu/modules/configuration/led_notify.ui"), this);
}

NotifierConfigurationWidget* LedNotify::createConfigurationWidget(QWidget*, char*)
{
	return 0;
}

void LedNotify::notify( Notification *notification )
{
	kdebugf();
	
	if( "NewChat" == notification->type() )
	{
		blinking_.startInfinite();
	}
	else
	{
		blinking_.startFinite();
	}
	kdebugf2();
}

void LedNotify::messageReceived( UserListElement user )
{
	kdebugf();
	if( !pending.pendingMsgs() )
	{
		blinking_.stop();
	}
	kdebugf2();
}
