/***************************************************************************
 *   Copyright (C) 2006 by bysk                                           *
 *   buysk@pf.pl                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "led_notify.h"
#include "chat_manager.h"
#include "main_configuration_window.h"
#include "configuration_window_widgets.h"
#include <qmessagebox.h>

#include <X11/Xlib.h>

int LedNotify::NUMLOCK_LED = 1;
int LedNotify::CAPSLOCK_LED = 2;
int LedNotify::SCROLLLOCK_LED = 3;

LedNotify* led_notify_session;

extern "C" int led_notify_init()
{
	kdebugf();
	led_notify_session = new LedNotify();
	kdebugf2();
	return 0;
}

extern "C" void led_notify_close()
{
	kdebugf();
	delete led_notify_session;
	led_notify_session = NULL;
	kdebugf2();
}

LedNotify::LedNotify(QObject* parent, const char* name)
:
	Notifier(parent, name),
	ledHighlight_( false ),
	blinking_( false ),
	stopBlinkingFromChat_( true ),
	stopBlinkingFromMsg_( true ),
	activeChat_( NULL )
{
	config_file.addVariable("Notify", "NewChat_LED", false);
	config_file.addVariable("Notify", "NewMessage_LED", false);
	MainConfigurationWindow::registerUiFile(dataPath("kadu/modules/configuration/led_notify.ui"), this);
	
	notification_manager->registerNotifier( "Scroll Lock LED", this );
	
	timer = new QTimer(this);
	connect( timer, SIGNAL(timeout()), this, SLOT(blink()));
	connect(chat_manager, SIGNAL(chatWidgetActivated(ChatWidget *)), this, SLOT(chatWidgetActivated(ChatWidget *)));
	connect( &pending, SIGNAL(messageFromUserDeleted(UserListElement)), this, SLOT(messageReceived(UserListElement)) );
}

LedNotify::~LedNotify()
{
	disconnect( &pending, SIGNAL(messageFromUserDeleted(UserListElement)), this, SLOT(messageReceived(UserListElement)) );
	disconnect(chat_manager, SIGNAL(chatWidgetActivated(ChatWidget *)), this, SLOT(chatWidgetActivated(ChatWidget *)));
	disconnect(timer, SIGNAL(timeout()), this, SLOT(blink()));
	notification_manager->unregisterNotifier("Scroll Lock LED");
	MainConfigurationWindow::unregisterUiFile(dataPath("kadu/modules/configuration/led_notify.ui"), this);
	
	if( ledHighlight_ )
		setLed( SCROLLLOCK_LED, false );
}

void LedNotify::mainConfigurationWindowCreated(MainConfigurationWindow*)
{
}

NotifierConfigurationWidget* LedNotify::createConfigurationWidget(QWidget*, char*)
{
	return 0;
}

void LedNotify::notify( Notification *notification )
{
	kdebugf();
	
	//QMessageBox::information( 0, "", "notify()" );
	
// 	if( !blinking_ )
// 	{
// 		blinking_ = true;
// 		stopBlinkingFromMsg_ = false;
// 		ledHighlight_ = false;
// 		
// 		timer->start(config_file.readNumEntry("Notify", "LEDdelay"));
// 		blink();
// 	}
	
	if( "NewChat" == notification->type() )
	{
		newChat();
	}
	else if( "NewMessage" == notification->type() )
	{
		newMessage( notification->userListElements() );
	}
	kdebugf2();
}

void LedNotify::newChat()
{
	kdebugf();
	
	if( !blinking_ )
	{
		blinking_ = true;
		stopBlinkingFromChat_ = false;
		ledHighlight_ = false;
		
		timer->start(config_file.readNumEntry("Notify", "LEDdelay"));
		blink();
	}
	kdebugf2();
}

void LedNotify::newMessage( const UserListElements& senders )
{
	kdebugf();
	
	ChatWidget* chat = chat_manager->findChatWidget( senders );
	if( chat != activeChat_ && chat != NULL )
	{
		msgChats_.push_back( chat );
	
		if( !blinking_ )
		{
			blinking_ = true;
			stopBlinkingFromMsg_ = false;
			ledHighlight_ = false;
			
			timer->start(config_file.readNumEntry("Notify", "LEDdelay"));
			blink();
		}
	}
	kdebugf2();
}

void LedNotify::blink(void)
{
	kdebugf();
	
	if( stopBlinkingFromChat_ && stopBlinkingFromMsg_ )
	//if( stopBlinkingFromChat_ && pending.pendingMsgs()	 )
	//if( !pending.pendingMsgs() )
	{
		timer->stop();
		blinking_ = false;
		if( !ledHighlight_ )
			return;
	}
	
	ledHighlight_ = !ledHighlight_;
	setLed( SCROLLLOCK_LED, ledHighlight_ );
	
	kdebugf2();
}

void LedNotify::setLed( int led, bool highlight )
{
	static Display* theDisplay = QPaintDevice::x11AppDisplay();
	static XKeyboardControl values;
	
	values.led = led;
	values.led_mode = (highlight ? LedModeOn : LedModeOff);

	XChangeKeyboardControl(theDisplay, KBLed | KBLedMode, &values);
}

void LedNotify::chatWidgetActivated( ChatWidget* chat )
{
	kdebugf();
	
	//QMessageBox::information( 0, "", "chatWidgetActivated()" );
	
	activeChat_ = chat;
	
	QValueList<ChatWidget*>::iterator chatIt = msgChats_.find( chat );
	if( chatIt != msgChats_.end() )
	{
		msgChats_.remove( chatIt );
	}
	
	if( msgChats_.empty() )
	{
		stopBlinkingFromMsg_ = true;
	}
	kdebugf2();
}

void LedNotify::messageReceived( UserListElement user )
{
	kdebugf();
	
	//QMessageBox::information( 0, "", "messageReceived()" );
	
	if( !pending.pendingMsgs() )
	{
		stopBlinkingFromChat_ = true;
	}
	kdebugf2();
}

void LedNotify::notificationClosed( Notification* )
{
	
}
